using System;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;

namespace SendMarkerDemo
{
    public static class Impl
    {
        private const uint WM_COPYDATA = 0x004A;

        [DllImport("user32.dll", CharSet = CharSet.Auto)]
        private static extern IntPtr SendMessage(IntPtr hWnd, UInt32 Msg, IntPtr wParam, IntPtr lParam);

        [StructLayout(LayoutKind.Sequential)]
        private struct COPYDATASTRUCT
        {
            public IntPtr dwData;
            public int cbData;
            public IntPtr lpData;
        }

        [DllImport("user32.dll", SetLastError = true)]
        private static extern IntPtr FindWindow(string lpClassName, string lpWindowName);

        public static void SendMarker(string text)
        {
            IntPtr serverWindowHandle = FindSVServerWindow();

            if (serverWindowHandle.ToInt32() != 0) 
            {
                IntPtr markerTextUnmanaged = IntPtr.Zero;
                int markerTextUnmanagedLength = 0;

                if (!string.IsNullOrEmpty(text)) {
                    byte[] buffer = Encoding.UTF8.GetBytes(text);
                    markerTextUnmanaged = Marshal.AllocHGlobal(buffer.Length);
                    Marshal.Copy(buffer, 0, markerTextUnmanaged, buffer.Length);
                    markerTextUnmanagedLength = buffer.Length;
                }

                COPYDATASTRUCT copyData = new COPYDATASTRUCT();
                copyData.dwData = IntPtr.Zero;
                copyData.lpData = markerTextUnmanaged;
                copyData.cbData = markerTextUnmanagedLength;

                IntPtr copyDataPtr = Marshal.AllocHGlobal(Marshal.SizeOf(typeof(COPYDATASTRUCT)));
                Marshal.StructureToPtr(copyData, copyDataPtr, false);

                SendMessage(serverWindowHandle, WM_COPYDATA, IntPtr.Zero, copyDataPtr);

                if (markerTextUnmanagedLength > 0)
                {
                    Marshal.FreeHGlobal(markerTextUnmanaged);
                }

                Marshal.FreeHGlobal(copyDataPtr);
            }
        }

        public static void SimulateReturnKey()
        {
            SendKeys.Send("~");
        }

        private static IntPtr FindSVServerWindow()
        {
            return FindWindow("wxWindowClassNR", "Somatic Vision Socket Server");
        }
    }
}
